/*
 changeuser.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2002      David Pinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gmasqdialer.h"
#include <unistd.h>

#ifndef __WIN32__
#include <pwd.h>
#endif

gboolean user_sent;


void menu_changeuser(void)
{

    GtkWidget *dialog;
    GtkWidget *table;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *entry_name, *entry_pass;
    GtkWidget *check;
    gint response;
    struct passwd *pw;

    /* ask user name & password */
    GtkWidget *vbox, *buthbox;
    GtkWidget *button;

    dialog = gtk_hig_dialog_new(NULL);
    gtk_window_set_title(GTK_WINDOW(dialog), "Change User");

    gtk_dialog_add_button(GTK_DIALOG(dialog), 
			  GTK_STOCK_CANCEL, 
			  GTK_RESPONSE_CANCEL);

    gtk_dialog_add_button(GTK_DIALOG(dialog), 
			  GTK_STOCK_OK, 
			  GTK_RESPONSE_OK);

    gtk_dialog_set_default_response (GTK_DIALOG(dialog),
				     GTK_RESPONSE_CANCEL);

    g_signal_connect(GTK_OBJECT(dialog), "delete_event",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);


    /* username */
    table = gtk_table_new(3, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
    gtk_container_set_border_width (GTK_CONTAINER(table), 6);

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
		       table, 
		       TRUE, 
		       TRUE, 
		       0);

    label = gtk_label_new(_("Username:"));
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);

    entry_name = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table), entry_name, 1, 2, 0, 1);
    gtk_object_set_data(GTK_OBJECT(dialog), "user", entry_name);

    /* set a default username, try the one in the config file, 
       otherwise use the currently logged in user 
    */
    if (username != NULL && *username != '\0')
	gtk_entry_set_text(GTK_ENTRY(entry_name), username);
    else
    {
#ifndef __WIN32__
        pw = getpwuid(getuid());
        gtk_entry_set_text(GTK_ENTRY(entry_name), pw->pw_name);
#endif
    }


    /* password */
    label = gtk_label_new(_("Password:"));
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);

    entry_pass = gtk_entry_new();
    gtk_entry_set_visibility(GTK_ENTRY(entry_pass), FALSE);
    gtk_table_attach_defaults(GTK_TABLE(table), entry_pass, 1, 2, 1, 2);

    gtk_object_set_data(GTK_OBJECT(dialog), "pass", entry_pass);

    /* default to the config file password */
    if (userpass != NULL && *userpass !='\0') 
	gtk_entry_set_text(GTK_ENTRY(entry_pass),userpass);
        
    /* checkbox for saving password to config file */
    check = gtk_check_button_new_with_label(_("Save Password?"));
    gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 2, 2, 3);

    gtk_object_set_data(GTK_OBJECT(dialog), "savepass", check);
    
    if (userpass != NULL && *userpass !='\0')
    	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),TRUE);

   
    /* finally */
    gtk_widget_show_all(dialog);

    /* modal is bad... */
    response = gtk_dialog_run(GTK_DIALOG(dialog));
    switch (response)
    {
    case GTK_RESPONSE_OK:
	mserver_setuser((gchar *) gtk_entry_get_text(GTK_ENTRY(entry_name)), NULL);
	mserver_setpass((gchar *) gtk_entry_get_text(GTK_ENTRY(entry_pass)), NULL);

	mserver_checkauth(callback_checkauth);
	user_sent = TRUE;

	break;
    case GTK_RESPONSE_CANCEL:
	break;
    default:
	/* printf("you pressed a button: 
	   but we didn't handle it: %d\n", response);*/
	break;
    }
    gtk_widget_destroy (dialog);

}
