/*
 setup.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2000-2002 David Pinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __WIN32__
#include <getopt.h>
#endif

#include "gmasqdialer.h"

gchar *masqhost;
gint masqport;
gboolean autoconnect;
gboolean autoredial;

gchar *username;
gchar *userpass;
gchar *autorun, *autorun_shutdown;
gboolean autosenduser;

gchar *defaultisp;

gboolean lastone_hangup;
gboolean show_mainwin_at_start;
gboolean hide_mainwin_after_connect;
gboolean show_menubar;
gboolean show_statusbar;
gboolean show_serverlog;
gboolean beep_connects;
gboolean panel_show_input, panel_show_output, panel_show_label;

void setup_init(void)
{
    masqhost = g_strdup(gnome_config_get_string("/"PACKAGE"/host/addr=localhost"));
    masqport = gnome_config_get_int("/"PACKAGE"/host/port=222");
    autoconnect = gnome_config_get_bool("/"PACKAGE"/host/autoconnect=FALSE");
    username = g_strdup(gnome_config_get_string("/"PACKAGE"/host/user="));
    userpass = g_strdup(gnome_config_get_string("/"PACKAGE"/host/pass="));
    autorun = g_strdup(gnome_config_get_string("/"PACKAGE"/host/autorun="));
    autorun_shutdown = g_strdup(gnome_config_get_string("/"PACKAGE"/host/autorun_shutdown="));
    autosenduser = gnome_config_get_bool("/"PACKAGE"/host/autosenduser=FALSE");
    lastone_hangup = gnome_config_get_bool("/"PACKAGE"/host/lasthangup=FALSE");
    show_mainwin_at_start = gnome_config_get_bool("/"PACKAGE"/host/showmainwin=TRUE");
    hide_mainwin_after_connect = gnome_config_get_bool("/"PACKAGE"/host/hidemainafterconnect=FALSE");
    show_menubar = gnome_config_get_bool("/"PACKAGE"/host/showmenubar=TRUE");
    show_statusbar = gnome_config_get_bool("/"PACKAGE"/host/showstatusbar=TRUE");
    show_serverlog = gnome_config_get_bool("/"PACKAGE"/host/showserverlog=TRUE");
    beep_connects = gnome_config_get_bool("/"PACKAGE"/host/beepconnects=FALSE");

    panel_show_input = gnome_config_get_bool("/"PACKAGE"/host/panelinput=TRUE");
    panel_show_output = gnome_config_get_bool("/"PACKAGE"/host/paneloutput=TRUE");
    panel_show_label = gnome_config_get_bool("/"PACKAGE"/host/panellabel=TRUE");
    autoredial = gnome_config_get_bool("/"PACKAGE"/host/autoredial=TRUE");
    defaultisp = g_strdup(gnome_config_get_string("/"PACKAGE"/host/isp="));
}


static void setup_apply(GtkWidget *dialog)
{
    printf("%s %d: Deprecated\n", __FILE__,__LINE__);
}


/*
 * Each of these *_changed call backs has a lot of duplication.
 * The intention is to carfully validate what the user inputs....but
 * Due to gtk not having a good way to deal with the user leaving the
 * entry, we write the stuff to disk, after every char. Not good. Only
 * a temp solution.......
 *
 */

static void masqaddr_changed(GtkWidget *entry, GtkWidget *dialog)
{
    gchar *name, *data;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(entry), "name");
    data = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    /* validate it !!! */
    masqhost = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_string(cfgpath, masqhost);
}

static void masqport_changed(GtkWidget *entry, GtkWidget *dialog)
{
    gchar *name, *data;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(entry), "name");
    data = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    /* validate it !!! */
    if ((sscanf(data, "%d", &masqport)) != 1) {
	//printf("%s %d: do something: we have an error\n", __FILE__,__LINE__);
	masqport=224;
    }

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_int(cfgpath, masqport);
}

static void user_entry_changed(GtkWidget *entry, GtkWidget *dialog)
{
    gchar *name, *data;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(entry), "name");
    data = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    /* validate it !!! */
    username = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_string(cfgpath, username);
}

static void autorun_entry_changed(GtkWidget *entry, GtkWidget *dialog)
{
    gchar *name, *data;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(entry), "name");
    data = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    /* validate it !!! */
    autorun = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_string(cfgpath, autorun);
}

static void autorun_shutdown_entry_changed(GtkWidget *entry, GtkWidget *dialog)
{
    gchar *name, *data;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(entry), "name");
    data = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);

    /* validate it !!! */
    autorun_shutdown = g_strdup(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_string(cfgpath, autorun_shutdown);
}


static gint value_changed(GtkWidget *widget, GtkWidget *dialog)
{
    printf("%s %d: Deprecated\n", __FILE__,__LINE__);
    return FALSE;
}

/** Toggle the checkbox, and update the config in the cfg file on disk
 *
 */
static void checkbox_clicked(GtkWidget *button, GtkWidget *dialog)
{
    gboolean flag;
    gchar *name;
    gchar *cfgpath;

    name = g_object_get_data(G_OBJECT(button), "name");
    flag = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), name));

    g_object_set_data(G_OBJECT(button), name, GINT_TO_POINTER(!flag));

    cfgpath = g_strdup_printf("/"PACKAGE"/host/%s", name);
    gnome_config_set_bool(cfgpath, !flag);
}

void create_checkbox(GtkWidget *dialog, 
		     GtkWidget *vbox, 
		     gchar *tag, 
		     gboolean flag, 
		     gchar *label)
{
    GtkWidget *button;

    button = gtk_check_button_new_with_label(label);
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), flag);

    g_object_set_data(G_OBJECT(button), "name", tag);
    g_signal_connect(GTK_OBJECT(button), "clicked",
		     G_CALLBACK(checkbox_clicked), dialog);
}

void menu_setup(void)
{
    GtkWidget *vbox_client, *vbox_server;
    GtkWidget *dialog;
    GtkWidget *addrentry, *portentry, *userentry;
    GtkWidget *autorunentry, *autorun_shutdownentry, *label;
    GtkWidget *notebook;
    GtkWidget *table;
    GString *str;
    gint response;
    
    dialog = gtk_dialog_new_with_buttons("Preferences", 
					 GTK_WINDOW(window), 
					 GTK_DIALOG_NO_SEPARATOR,
					 GTK_STOCK_CLOSE,
					 GTK_RESPONSE_NONE,
					 NULL);

    g_signal_connect_swapped (GTK_OBJECT (dialog), 
			      "response", 
			      G_CALLBACK (gtk_widget_destroy),
			      GTK_OBJECT (dialog));

    /* Client tab */
    vbox_client = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_client), 6);
    
    /* Server tab */
    vbox_server = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_server), 6);

    /* fillout server */
    table = gtk_table_new(5, 2, FALSE);

    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
    gtk_container_set_border_width (GTK_CONTAINER(table), 6);

    gtk_box_pack_start(GTK_BOX(vbox_server), table, TRUE, TRUE, 0);


    /* server address */
    label = gtk_hig_label_new(_("Masqdialer server address: "));
    gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 0, 1);

    addrentry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table),addrentry, 1, 2, 0, 1);

    if (masqhost != NULL) gtk_entry_set_text(GTK_ENTRY(addrentry), masqhost);

    g_object_set_data(G_OBJECT(addrentry), "name", "addr");
    g_signal_connect (GTK_OBJECT (addrentry), "changed",
		      GTK_SIGNAL_FUNC (masqaddr_changed), dialog);

    /* server port */
    label = gtk_hig_label_new(_("Masqdialer server port: "));
    gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 1, 2);

    portentry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table),portentry, 1, 2, 1, 2);

    str = g_string_new(NULL);
    g_string_printf(str, "%d", masqport);
    gtk_entry_set_text(GTK_ENTRY(portentry), str->str);
    g_string_free(str, TRUE);

    g_object_set_data(G_OBJECT(portentry), "name", "port");
    g_signal_connect (GTK_OBJECT (portentry), "changed",
		      GTK_SIGNAL_FUNC (masqport_changed), dialog);

    /* default user name */
    label = gtk_hig_label_new(_("Default user name: "));
    gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 2, 3);

    userentry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table),userentry, 1, 2, 2, 3);

    if (username != NULL) gtk_entry_set_text(GTK_ENTRY(userentry), username);

    g_object_set_data(G_OBJECT(userentry), "name", "user");
    g_signal_connect (GTK_OBJECT (userentry), "changed",
		      GTK_SIGNAL_FUNC (user_entry_changed), dialog);

    /* Run command after conn entry */
    label = gtk_hig_label_new(_("Run command after connected: "));
    gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 3, 4);

    autorunentry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table),autorunentry, 1, 2, 3, 4);

    if (autorun != NULL) gtk_entry_set_text(GTK_ENTRY(autorunentry), autorun);

    g_object_set_data(G_OBJECT(autorunentry), "name", "autorun");
    g_signal_connect (GTK_OBJECT (autorunentry), "changed",
		      GTK_SIGNAL_FUNC (autorun_entry_changed), dialog);

    /* Run command before hangup entry */
    label = gtk_hig_label_new(_("Run command before hang-up: "));
    gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 4, 5);

    autorun_shutdownentry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table),autorun_shutdownentry, 1, 2, 4, 5);

    if (autorun_shutdown != NULL) gtk_entry_set_text(GTK_ENTRY(autorun_shutdownentry), autorun_shutdown);

    g_object_set_data(G_OBJECT(autorun_shutdownentry), "name", "autorun_shutdown");
    g_signal_connect (GTK_OBJECT (autorun_shutdownentry), "changed",
		      GTK_SIGNAL_FUNC (autorun_shutdown_entry_changed), dialog);

    /* check boxes */
    create_checkbox(dialog, vbox_server, "autoconnect", autoconnect, _("Autoconnect to server at startup: "));
    create_checkbox(dialog, vbox_server, "autosenduser", autosenduser, _("Send user information to server at startup: "));
    create_checkbox(dialog, vbox_server, "lasthangup", lastone_hangup, _("'Last client hangs up' is set on server: "));

    /* fillout client */
    create_checkbox(dialog, vbox_client, "showmainwin", show_mainwin_at_start, _("Show main window at startup"));
    create_checkbox(dialog, vbox_client, "hidemainafterconnect", hide_mainwin_after_connect, _("Hide main window after connected"));
    create_checkbox(dialog, vbox_client, "showmenubar", show_menubar, _("Show menubar"));
    create_checkbox(dialog, vbox_client, "showstatusbar", show_menubar, _("Show statusbar"));
    create_checkbox(dialog, vbox_client, "showserverlog", show_serverlog, _("Show masqdialer server log"));
    create_checkbox(dialog, vbox_client, "beepconnects", beep_connects, _("Beep when connected/disconnected"));
    create_checkbox(dialog, vbox_client, "panellabel", panel_show_label, _("Show connection time in applet"));
    create_checkbox(dialog, vbox_client, "panelinput", panel_show_input, _("Show input load meter in applet"));
    create_checkbox(dialog, vbox_client, "paneloutput", panel_show_output, _("Show output load meter in applet"));


    notebook = gtk_notebook_new();
    // Hide client tab for now (without applet, it makes no sense anyway...)
    //gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_client, gtk_label_new(_("Client")));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_server, gtk_label_new(_("Server")));

    gtk_container_set_border_width(GTK_CONTAINER(GTK_NOTEBOOK(notebook)), 6);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);

    gtk_widget_show_all(dialog);

}
